-- Saone RaidFrames oUF layout by Saone of Llane, forked from Xenios Bretheren RaidFrames

assert(oUF, "oUF is not installed?")

-- Config variables
-- ---------------------
local raidHeight = 26
local raidWidth = 48
local powerWidth = 6
local iconsize = 12
local fontsize = 10
local symbolsize = 8
local indicatorsize = 6

-- ---------------------

-- ***************
-- * Helper Functions / Variables 
-- ***************

-- Power colors
oUF.colors.power['MANA'] = {0, 144/255, 1}

-- The backdrop texture
local backdrop = {
	bgFile = [=[Interface\ChatFrame\ChatFrameBackground]=], tile = true, tileSize = 16,
	insets = {top = -1, left = -1, bottom = -1, right = -1},
}

-- Unit Menu
local dropdown = CreateFrame("Frame", "MyAddOnUnitDropDownMenu", UIParent, "UIDropDownMenuTemplate")

UIDropDownMenu_Initialize(dropdown, function(self)
	local unit = self:GetParent().unit
	if not unit then return end

	local menu, name, id
	if UnitIsUnit(unit, "player") then
		menu = "SELF"
	elseif UnitIsUnit(unit, "vehicle") then
		menu = "VEHICLE"
	elseif UnitIsUnit(unit, "pet") then
		menu = "PET"
	elseif UnitIsPlayer(unit) then
		id = UnitInRaid(unit)
		if id then
			menu = "RAID_PLAYER"
			name = GetRaidRosterInfo(id)
		elseif UnitInParty(unit) then
			menu = "PARTY"
		else
			menu = "PLAYER"
		end
	else
		menu = "TARGET"
		name = RAID_TARGET_ICON
	end
	if menu then
		UnitPopup_ShowMenu(self, menu, unit, name, id)
	end
end, "MENU")

local menu = function(self)
	dropdown:SetParent(self)
	ToggleDropDownMenu(1, nil, dropdown, "cursor", 0, 0)
end

-- Used to find out when to display dispellable debuffs
-- local _, class = UnitClass("player")
-- local dispellClass = {
	-- PRIEST = { Magic = true, Disease = true, },
	-- SHAMAN = { Curse = true, Magic = true, },
	-- PALADIN = { Magic = true, Poison = true, Disease = true, },
	-- MAGE = { Curse = true, },
	-- DRUID = { Curse = true, Poison = true, Magic = true, },
-- }
-- local dispellist = dispellClass[class] or {}
-- local dispellPriority = {
	-- Magic = 4,
	-- Poison = 3,
	-- Curse = 2,
	-- Disease = 1,
-- }

-- ***************
-- * oUF Tags
-- ***************

-- Displays the name of the unit. If the unit has aggro, their name turns red.
oUF.Tags['sRname'] = function(u, r)
	if(not UnitExists(u)) then return '' end 
	return string.format("%0.7s", UnitName(u or r)) or ''
end
oUF.TagEvents['sRname'] = 'UNIT_NAME_UPDATE PLAYER_ENTERING_WORLD UNIT_THREAT_LIST_UPDATE UNIT_THREAT_SITUATION_UPDATE PARTY_MEMBERS_CHANGED RAID_ROSTER_UPDATE'


-- Displays the amount of health missing from the unit.
oUF.Tags['sRhealth'] = function(u)
	if (not UnitIsConnected(u)) or (not UnitExists(u)) then 
		return ''
	end

	local curr, max = UnitHealth(u), UnitHealthMax(u)

	if UnitIsDeadOrGhost(u) then
		return 'Dead'
	elseif curr/max > 0.9 then
		return ''
	else
		return format("-%0.1f", math.floor((max - curr)/100)/10)
	end	
end
oUF.TagEvents['sRhealth'] = 'UNIT_HEALTH UNIT_MAXHEALTH'

-- ***************
-- * Update Functions 
-- ***************

-- Shows / Hides the focus border on a unit
local FocusTarget = function(self)
	if UnitIsUnit('focus', self.unit) then
		self.FocusHighlight:SetBackdropColor(.6, .8, 0, 1)
		self.FocusHighlight:Show()
		self.Border:Hide()
	else
		self.FocusHighlight:Hide()
		self.Border:Show()
	end
end

-- Show tooltips
local OnEnter = function(self)
    UnitFrame_OnEnter(self)
end

local OnLeave = function(self)
    UnitFrame_OnLeave(self)
end

-- ***************
-- * oUF Styles / Initalization
-- ***************

-- The main style , a recreation of my main rdx frames
local myStyle = function(self)
	self.menu = menu
		
	self:SetBackdrop(backdrop)
	self:SetBackdropColor(0, 0, 0, 1)

	self:SetFrameLevel(5)

	--Register for clicks, never use anyup.
	self:RegisterForClicks"AnyDown"
	self:SetAttribute("*type2", "menu")

	--Tooltips!
    self:SetScript("OnEnter", OnEnter)
    self:SetScript("OnLeave", OnLeave)
	
	-- Health bar
	self.Health = CreateFrame('StatusBar', nil, self)
	self.Health:SetFrameLevel(6) -- Makes the hp frame lower than the highlight frame. Magic!
	self.Health:SetPoint('TOPLEFT', self, 'TOPLEFT', 0, 0)
	self.Health:SetPoint('BOTTOMRIGHT', self, 'BOTTOMRIGHT', 0 - powerWidth, 0)
	self.Health:SetOrientation('VERTICAL')
	self.Health:SetStatusBarTexture([=[Interface\AddOns\oUF_sRaid\media\smooth]=])
	self.Health.colorDisconnected = true
	self.Health.colorClass = true
	self.Health.frequentUpdates = true
	
	self.Health.bg = self.Health:CreateTexture(nil, 'BORDER')
	self.Health.bg:SetAllPoints(self.Health)
	self.Health.bg:SetTexture([=[Interface\ChatFrame\ChatFrameBackground]=])
	self.Health.bg.multiplier = 0.3

	-- Unit name text
	local name = self.Health:CreateFontString(nil, 'OVERLAY', 'GameFontHighlightExtraSmall')
	name:SetPoint('LEFT', self.Health, 'LEFT', -3, 6)
	name:SetPoint('RIGHT', self.Health, 'RIGHT', 3, 6)
	name:SetTextColor( .8, .8, .8)
	name.overrideUnit = true
	self:Tag(name, "[threatcolor][sRname]")
	
	-- Unit health text
	local health = self.Health:CreateFontString(nil, 'OVERLAY', 'GameFontHighlightSmallRight')
	health:SetPoint('CENTER', self.Health, 'CENTER', 0, -6)
	health:SetTextColor( .8, .8, .8)
	health:SetJustifyH("CENTER")
	health:SetJustifyV("CENTER")
	health.overrideUnit = true

	self:Tag(health, "[sRhealth] ")
	
	-- Power bar
	self.Power = CreateFrame('StatusBar', nil, self)
	self.Power:SetPoint('TOPRIGHT', self, 'TOPRIGHT', 0, 0)
	self.Power:SetPoint('BOTTOMLEFT', self.Health, 'BOTTOMRIGHT', 1, 0)
	self.Power:SetOrientation('VERTICAL')
	self.Power:SetStatusBarTexture([=[Interface\AddOns\oUF_sRaid\media\manatex]=])
	self.Power.colorDisconnected = true
	self.Power.colorPower = true

	-- Power frame background
	self.Power.bg = self.Power:CreateTexture(nil, 'BORDER')
	self.Power.bg:SetAllPoints(self.Power)
	self.Power.bg:SetTexture([=[Interface\ChatFrame\ChatFrameBackground]=])
	self.Power.bg.multiplier = 0.3
		
	-- Highlight tex
	local hl = self.Health:CreateTexture(nil, "OVERLAY")
	hl:SetAllPoints(self)
	hl:SetTexture([=[Interface\ChatFrame\ChatFrameBackground]=])
	hl:SetVertexColor(1,1,1,.1)
	hl:SetBlendMode("ADD")
	hl:Hide()
	self.Highlight = hl

	-- always on border
	local Border = CreateFrame("Frame", nil, self)
	Border:SetPoint("TOPLEFT", self, "TOPLEFT")
	Border:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT")
	Border:SetBackdrop(border)
	-- Border:SetBackdropColor(ns.settings.borderColor[1],ns.settings.borderColor[2],ns.settings.borderColor[3])
	Border:SetFrameLevel(2)
	Border:Show()
	self.Border = Border
	
	-- Focus Border
	local fBorder = CreateFrame("Frame", nil, BorderFrame1)
	fBorder:SetPoint("TOPLEFT", self, "TOPLEFT")
	fBorder:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT")
	fBorder:SetBackdrop(border)
	fBorder:Hide()
	self.FocusHighlight = fBorder
	
	-- Dispel Border
	local dispelBorder = CreateFrame("Frame", nil, BorderFrame1)
	dispelBorder:SetPoint("TOPLEFT", self, "TOPLEFT")
	dispelBorder:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT")
	dispelBorder:SetBackdrop(border)
	dispelBorder:Hide()
	self.dispelBorder = dispelBorder
	
	-- Raid Icons
	local ricon = self.Health:CreateTexture(nil, 'OVERLAY')
	ricon:SetPoint("TOP", self, 0, 5)
	ricon:SetSize(iconsize, iconsize)
	self.RaidIcon = ricon
	
	-- Leader Icon
	self.Leader = self.Health:CreateTexture(nil, "OVERLAY")
	self.Leader:SetPoint("TOPLEFT", self, 0, 8)
	self.Leader:SetSize(iconsize, iconsize)

	-- Assistant Icon
	self.Assistant = self.Health:CreateTexture(nil, "OVERLAY")
	self.Assistant:SetPoint("TOPLEFT", self, 0, 8)
	self.Assistant:SetSize(iconsize, iconsize)

	-- Master looter Icon
	local masterlooter = self.Health:CreateTexture(nil, 'OVERLAY')
	masterlooter:SetSize(iconsize, iconsize)
	masterlooter:SetPoint('LEFT', self.Leader, 'RIGHT')
	self.MasterLooter = masterlooter
	
	-- LFD Icon
	self.LFDRole = self.Health:CreateTexture(nil, 'OVERLAY')
	self.LFDRole:SetSize(iconsize, iconsize)
	self.LFDRole:SetPoint('RIGHT', self, 'LEFT', iconsize/2, iconsize/2)

	-- Mouseover Highlighting
	self.Highlight = self:CreateTexture(nil, "HIGHLIGHT") 	
	self.Highlight:SetTexture(.2, .2, .2, .7)
	self.Highlight:SetBlendMode("ADD")
	self.Highlight:SetAllPoints(self)

	-- ReadyCheck Icon
	self.ReadyCheck = self.Health:CreateTexture(nil, "OVERLAY")
	self.ReadyCheck:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT")
	self.ReadyCheck:SetSize(iconsize, iconsize)
	self.ReadyCheck.delayTime = 8
	self.ReadyCheck.fadeTime = 1

	-- Range
	self.Range = {
		insideAlpha = 1,
		outsideAlpha = 0.4,
	}

	-- Add events
	self:RegisterEvent('PLAYER_FOCUS_CHANGED', FocusTarget)
	self:RegisterEvent('RAID_ROSTER_UPDATE', FocusTarget)
	self:RegisterEvent('PLAYER_ENTERING_WORLD', getzone)
end

oUF:RegisterStyle("sRaid", myStyle)

-- Hide the blizzard frames
CompactRaidFrameManager:UnregisterAllEvents()
CompactRaidFrameManager:Hide()
CompactRaidFrameContainer:UnregisterAllEvents()
CompactRaidFrameContainer:Hide() 
CompactRaidFrameContainer:Hide()

-- Spawns the various windows if needed
oUF:Factory(function(self)
	self:SetActiveStyle"sRaid"
		
	local visible = 'raid,party,solo'
	
	-- Raid
	raid = {}
	for i = 1, 8 do 
		local group = self:SpawnHeader('oUF_Raid'..i, nil, visible,
		'oUF-initialConfigFunction', ([[
		self:SetWidth(%d)
		self:SetHeight(%d)
		]]):format(50, 26),
		'showPlayer', true,
		'showSolo', true,
		'showParty', false,
		'showRaid', true,
		'xoffset', 6,
		'yOffset', 0,
		'point', "LEFT",
		'groupFilter', tostring(i),
		'groupingOrder', '1,2,3,4,5,6,7,8',
		'groupBy', 'GROUP',
		'maxColumns', 5,
		'unitsPerColumn', 5,
		'columnSpacing', 3,
		'columnAnchorPoint', "LEFT"
		)
		if i == 1 then
			group:SetPoint('BOTTOMRIGHT', UIParent, 'BOTTOMRIGHT', -6, 6)
		else
			group:SetPoint('BOTTOMRIGHT', raid[i - 1], 'TOPRIGHT', 0, 6)
		end
		group:SetScale(1)
		raid[i] = group
	end
end)

-- addon load, I guess I dont really need this anymore.
local f = CreateFrame("Frame")
f:RegisterEvent("ADDON_LOADED")
f:SetScript("OnEvent", function(self, event, addon)
	if addon ~= ADDON_NAME then return end

	self:UnregisterEvent("ADDON_LOADED")
end)
